var popupDialogPort, noteStore;

chrome.runtime.onConnect.addListener( function(receivedPort) {

	console.log("Received Incoming Port connection from popup.js: " + receivedPort.name);
	popupDialogPort = receivedPort;

  	popupDialogPort.onMessage.addListener(function(msg) {    
		console.log("Received msg from popup dialog: " + msg.type);

		switch(msg.type) {
			case 'init': initialize(msg); break;
			case "syncNotes": syncNotes(msg); break;
			case 'purchaseLicense': purchaseLicense(msg); break;
		}
	});

	popupDialogPort.onDisconnect.addListener( function(disconnectedPort) {
		console.log('Received port disconnect event from backgound.js: ' + disconnectedPort.name);
		popupDialogPort = null;
		syncNotes();
	});

});

function sendMessage(msg) {
	if(popupDialogPort) {
		console.log('Sent message to popup script: ' + msg.type);
		popupDialogPort.postMessage(msg);
	} else {
		//console.log('Cannot send msg to popup dialog, port disconnected');
	}
}


function replaceAllSubstrings(string, substring, newSubstring) {
    while(string.indexOf(substring) > -1) {
        string = string.replace(substring,newSubstring);
    }
    return string;
}

function cleanHtmlTags(content) {
	var brRegex = /<br\s*[clear="none"]*\/*>/g;
	var cleanContent = content.replace(brRegex, '<br/>');
	return cleanContent;
}

function cleanTodoElements(content) {
	var cleanContent;
	var tempDom = $('<div>').append($.parseHTML(content));
	var toDos = $(tempDom[0]).find("input.en-todo-image");

	_.each(toDos, function(toDo, index, list) {
		var isChecked = $(toDo).attr('data-checked') === 'true';
		(isChecked) ? $(toDo).replaceWith('<en-todo checked="true"></en-todo>') : $(toDo).replaceWith('<en-todo checked="false"></en-todo>');
	});
	cleanContent = $(tempDom).html();
	
	return cleanContent;
}

function corectBodyNote(content) {
	//console.log('pre content: ' + content);
    var cleanContent = replaceAllSubstrings(content,'<en-note>','');
    cleanContent = replaceAllSubstrings(cleanContent, '</en-note>','');
    cleanContent = cleanTodoElements(cleanContent);
    cleanContent = cleanHtmlTags(cleanContent);
    //console.log('post content: ' + cleanContent);
    return cleanContent;
}

function initialize(msg) {
	if(!noteStore) {
		noteStoreURL = localStorage['noteStoreUrl'];
		noteStoreTransport = new Thrift.BinaryHttpTransport(noteStoreURL);
		noteStoreProtocol = new Thrift.BinaryProtocol(noteStoreTransport);
		noteStore = new NoteStoreClient(noteStoreProtocol);
	}
}

function syncNotes(msg) {
	var authenticationToken = localStorage['token'];
	var isNewNote = msg ? msg.isNewNote : (localStorage['isNewNote'] === "true");

	console.log('Saving notes to evernote. localStorage["isNewNote"]: ' + localStorage['isNewNote'] + ', isNewNote: ' + isNewNote);

	if(isNewNote) {

		var textContent = localStorage['oldNoteContent'].replace('<!--?xml version="1.0" encoding="UTF-8"?-->', '');
		
		var nBody = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
		nBody += "<!DOCTYPE en-note SYSTEM \"http://xml.evernote.com/pub/enml2.dtd\">";
		nBody += "<en-note>" + corectBodyNote(textContent) + "</en-note>";
		
		var newNote = new Note();
		newNote.title = localStorage['oldNoteTitle'];
		newNote.content = nBody;
		newNote.contentLength = nBody.length;
		newNote.notebookGuid =localStorage['oldNoteNotebook'];

		noteStore.createNote(authenticationToken, newNote, function(nt) {
			if(nt && nt.guid) {
				localStorage['lastOpenedNote'] = nt.guid;
			}

			if(localStorage['unsaved'] == 'true') {
				localStorage['unsaved'] = 'false';
			}
			localStorage['isNewNote'] = false;

			console.log('New note synced to evernote');

			sendMessage({ type: 'notesSynced', savedNote: nt });
		});
	}
	else {
		
		noteStore.getNote(authenticationToken, localStorage['oldNoteGuid'], true, true, true, true, function(crtNote){
			
			crtNote.title = localStorage['oldNoteTitle'];
			var nBody = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
			 nBody += "<!DOCTYPE en-note SYSTEM \"http://xml.evernote.com/pub/enml2.dtd\">";
			 nBody +="<en-note>";

			var doc = new DOMParser().parseFromString(localStorage['oldNoteContent'].replace('<en-note>','').replace('</en-note>',''), 'text/html');
			var result = new XMLSerializer().serializeToString(doc);
			result = result.replace('<html xmlns="http://www.w3.org/1999/xhtml"><head></head><body>','').replace('</body></html>','');
			result = replaceAllSubstrings(result, '<!--?xml version="1.0" encoding="UTF-8"?-->','');
            nBody += corectBodyNote(result);

			 nBody +="</en-note>";

			crtNote.content = nBody;
			crtNote.updated = (new Date()).getTime();
			noteStore.updateNote(authenticationToken, crtNote, function(newSavedNote) {
				if(newSavedNote.errorCode) {
					console.error('Error while syncing notes: ' + newSavedNote.parameter);
				} else {
					if(localStorage['unsaved'] == 'true') {
						localStorage['unsaved'] = 'false';
					}
					console.log('Updated note synced to evernote');

					sendMessage({ type: 'notesSynced', savedNote: newSavedNote });
				}
			});
		});
	}
}

function onPurchase(purchase) {
  //console.log("onPurchase", purchase);
  var jwt = purchase.jwt;
  var cartId = purchase.request.cardId;
  var orderId = purchase.response.orderId;

}

function onPurchaseFailed(purchase) {
  //console.log("onPurchaseFailed", purchase);
  var reason = purchase.response.errorType;
}


function purchaseLicense() {
	var sku = "everpremium";

	google.payments.inapp.buy({
    	parameters: {'env': "prod"},
    	'sku': sku,
    	'success': onPurchase,
    	'failure': onPurchaseFailed
  	});
}
